class EmoState {

  int id = -1;
  String userText = "";

  // Width of the square emoticon
  int emoSize = 4;

  // 0 = happy
  // 1 = sad
  // 2 = angry
  // 3 = disgusted
  // 4 = surprised
  // 5 = afraid
  int[] emotions = new int[6];

  // Which emotions to show most powerfully
  int first = -1;
  int sec = -1;
  int third = -1;

  PImage eyes = null;
  PImage mouth = null;

  EmoState() {
    randomEmotion();
  }

  EmoState(int ID) {
    setEmoID(ID, "");
    randomEmotion();
  }

  EmoState(int ID, String userText, int hap, int sad, int ang, int dis, int sur, int afr) {
    setEmoID(ID, userText);
    setEmotion(hap, sad, ang, dis, sur, afr);
  }

  EmoState(int ID, String userText, int hap, int sad, int ang, int dis, int sur, int afr, PImage e, PImage m) {
    setEmoID(ID, userText);
    setEmotion(hap, sad, ang, dis, sur, afr);
    setImages(e, m);
  }

  void randomEmotion() {
    // To make a variety of strengths
    int maxEmoStr = ((int) random(5)) +1;
    println("RANDOM arbitrary max emotion strength: " + maxEmoStr);
    for (int i = 0; i < emotions.length; i++) {
      emotions[i] = ((int) random(maxEmoStr)) + 1;
      println("emotion " + i + " set to " + emotions[i]);
    }
    configureFace();
  }

  void configureFace() {
    ArrayList[] ranks = new ArrayList[emotions.length];
    // Initialize all these array Lists
    for (int i = 0; i < ranks.length; i++) {
      ranks[i] = new ArrayList();
    }
    for (int i = 0; i < emotions.length; i++) {
      // rank index = value of emotion, add the index of the emotion if it matches
      ranks[emotions[i]].add(i);
    }
    // Sort through the ranks to find the highests
    for (int i = ranks.length-1; i >= 0; i--) {
      int[] tempRanks = new int[3]; // indexes of top-ranking emotions
      tempRanks[0] = tempRanks[1] = tempRanks[2] = -1;
      if (ranks[i].size() >= 3) {
        // There are many options, pick 3 of them and randomize
        for (int j = 0; j < 3; j++) {
          int randIndex = (int) random(ranks[i].size());
          tempRanks[j] = (Integer)ranks[i].get(randIndex);
          //println("("+ i + ") tmpRank " + j + " set to index " + tempRanks[j]);
          ranks[i].remove(randIndex);
        }
      } 
      else if (ranks[i].size() == 2) {
        // There are two options, randomize their importance
        for (int j = 0; j < 2; j++) {
          int randIndex = (int) random(ranks[i].size());
          tempRanks[j] = (Integer)ranks[i].get(randIndex);
          //println("("+ i + ") tmpRank " + j + " set to index " + tempRanks[j]);
          ranks[i].remove(randIndex);
        }
      } 
      else if (ranks[i].size() == 1) {
        // There is one option, maximize its importance
        tempRanks[0] = (Integer)ranks[i].get(0);
        //println("("+ i + ") tmpRank " + 0 + " set to index " + tempRanks[0]);
      } 
      else {
        // Do nothing and move onto the next rank
      }
      for (int j = 0; j < tempRanks.length; j++) {
        if (first == -1 && tempRanks[j] != -1) {
          first = tempRanks[j];
          println("set first rank " + j + ": " + tempRanks[j] + " at value " + emotions[tempRanks[j]]);
        }
        else if (sec == -1 && tempRanks[j] != -1) {
          sec = tempRanks[j];
          println("set second rank " + j + ": " + tempRanks[j] + " at value " + emotions[tempRanks[j]]);
        } 
        else if (third == -1 && tempRanks[j] != -1) {
          third = tempRanks[j];
          println("set third rank " + j + ": " + tempRanks[j] + " at value " + emotions[tempRanks[j]]);
        }
      }
    }
    // Once it's all over, set the images!
    loadImages();
  }

  void loadImages() {
    String eyeWords = "eyes_";
    String mouthWords = "mouth_";

    // Check if one emotion is accutely stronger than the others
    if (emotions[first] > 2 && emotions[first] - emotions[sec] >= 3) {
      println("-- Replacing sec emotion (" + emotions[sec] + ") with the first");
      sec = first;
    }

    // Pull out the files
    switch(first) {
      case(0):
      eyeWords += "happy";
      break;
      case(1):
      eyeWords += "sad";
      break;
      case(2):
      eyeWords += "angry";
      break;
      case(3):
      eyeWords += "disgusted";
      break;
      case(4):
      eyeWords += "surprised";
      break;
      case(5):
      eyeWords += "afraid";
      break;
    default:
      break;
    }
    switch(sec) {
      case(0):
      mouthWords += "happy";
      break;
      case(1):
      mouthWords += "sad";
      break;
      case(2):
      mouthWords += "angry";
      break;
      case(3):
      mouthWords += "disgusted";
      break;
      case(4):
      mouthWords += "surprised";
      break;
      case(5):
      mouthWords += "afraid";
      break;
    default:
      break;
    }

    eyeWords += "_"+ emotions[first];
    mouthWords += "_" + emotions[sec];
    println("LOADING: " + eyeWords + ".png");
    println("LOADING: " + mouthWords + ".png");
    setImages(loadImage(eyeWords + ".png"), loadImage(mouthWords + ".png"));
  }

  // 15 x 15 is the total size of the composite emoticon
  void setImages(PImage eyes, PImage mouth) {
    this.eyes = eyes;
    eyes.resize(emoSize * 15, 0);
    this.mouth = mouth;
    mouth.resize(emoSize * 15, 0);
  }

  void setEmotion(int hap, int sad, int ang, int dis, int sur, int afr) {
    emotions[0] = hap;
    emotions[1] = sad;
    emotions[2] = ang;
    emotions[3] = dis;
    emotions[4] = sur;
    emotions[5] = afr;
    configureFace();
  }

  void setEmoID(int value, String userText) {
    this.id = value;
    this.userText = userText;
  }
  
  void resize(int eSizeInt){
    emoSize = eSizeInt;
    loadImages();
  }

  void drawState(int x, int y) {
    int imageX = x - ((emoSize*15)/2);
    int imageY = y - ((emoSize*15)/2);
    if (eyes != null) {
      image(eyes, imageX, imageY);
    }
    if (mouth != null) {
      image(mouth, imageX, imageY + eyes.height);
    }
  }

  EmoState copyCurrState() {
    return new EmoState(id, userText, emotions[0], emotions[1], emotions[2], emotions[3], emotions[4], emotions[5], eyes, mouth);
  }
}

