import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class Emotale3 extends PApplet {

Character pc;
ArrayList npcs;

Boolean showKeyboard = false;
Character cCollided;

PImage keyboardG;
String workingEmoticon = "";
PFont font;

// 0 = moving
// 1 = typing
int MODE = 0;

public void setup() {
  size(1000, 600);
  pc = new Character("pc");

  npcs = new ArrayList();
  int randChars = (int) random(7) + 1;
  for (int i = 0; i < randChars; i++) {
    int randomX = (int) random(width-200) + 100;
    int randomY = (int) random(height-200) + 100;
    Character npc = new Character("npc" + i, randomX, randomY);
    npcs.add(npc);
  }
  keyboardG = loadImage("keyboard.png");
  keyboardG.resize(pc.state.emoSize * 15, 0);
  font = loadFont("Gisha-12.vlw");
}

public void update() {
  if (keyPressed && MODE == 0) {
    if (key == CODED) {
      if (keyCode == UP) moveCharacter(pc, 0);
      if (keyCode == RIGHT) moveCharacter(pc, 1);
      if (keyCode == DOWN) moveCharacter(pc, 2);
      if (keyCode == LEFT) moveCharacter(pc, 3);
    } 
    else {
      if (key == 'w') moveCharacter(pc, 0);
      if (key == 'd') moveCharacter(pc, 1);
      if (key == 's') moveCharacter(pc, 2);
      if (key == 'a') moveCharacter(pc, 3);
    }
  }
  pc.updateCharacter();

  // Check for which character to interact with
  Boolean collisionD = false;
  cCollided = null;
  for (int i = 0; i < npcs.size(); i++) {
    Character c = (Character) npcs.get(i);
    c.updateCharacter();

    // if the player comes within 20 pixels
    if (pc.checkColl(c.collX - 10, c.collY-10, c.wid + 20, c.wid + 20)) {
      collisionD = true;
      cCollided = c;
      //println("collided with " + c.name);
      targetC = c;
    }
  }

  // Show the keyboard graphic if in range to type
  if (collisionD == true){
    showKeyboard = true;
  } else {
    showKeyboard = false;
    targetC = null;
  }
}

public void draw() {
  update();
  background(30);
  pc.drawCharacter();
  noFill();
  stroke(0, 255, 0);
  rect(pc.collX, pc.collY, pc.wid, pc.wid);

  for (int i = 0; i < npcs.size(); i++) {
    Character c = (Character) npcs.get(i);
    c.drawCharacter();
    noFill();
    stroke(255, 0, 0);
    //rect(c.collX - 10, c.collY-10, c.wid + 20, c.wid + 20);
  }

  if (showKeyboard) {
    image(keyboardG, pc.collX, pc.collY + pc.wid + 30);
  }
  // Show textbox & text
  if (MODE == 1){
    fill (255);
    stroke(0);
    rect(pc.collX, pc.collY + pc.wid + 5, pc.wid, 20);
    fill(0);
    textFont(font);
    text(workingEmoticon, pc.collX + 4, pc.collY + pc.wid + 19);
  }
}


public void moveCharacter(Character c, int direction) {
  //println("Moving " + c.name + " " + direction);
  switch(direction) {
  case 0:
    c.ypos -= c.speed;
    break;
  case 1:
    c.xpos += c.speed;
    break;
  case 2:
    c.ypos += c.speed;
    break;
  case 3:
    c.xpos -= c.speed;
    break;
  default:
    break;
  }
}

public void reset() {
  pc = new Character("pc");

  npcs = new ArrayList();
  int randChars = (int) random(7) + 1;
  for (int i = 0; i < randChars; i++) {
    int randomX = (int) random(width-200) + 100;
    int randomY = (int) random(height-200) + 100;
    Character npc = new Character("npc" + i, randomX, randomY);
    npcs.add(npc);
  }
}

public void keyPressed() {
  if (MODE == 0) {
    if (key == 'r') reset();
    if (keyCode == ENTER) {
      MODE = 1;
      workingEmoticon = "";
    }
  } 
  else if (MODE == 1) {
    if (keyCode == ENTER || keyCode == RETURN){
      parseInput();
      MODE = 0;
      workingEmoticon = "";
    }
    if (keyCode == ESC) {
      MODE = 0;
      workingEmoticon = "";
    }
    if (keyCode == BACKSPACE && workingEmoticon != ""){
      println("BACKSPACE");
      workingEmoticon = workingEmoticon.substring(0, workingEmoticon.length()-1);
    } else if (key != CODED && key != SHIFT && keyCode != SHIFT && key != ENTER && keyCode != ENTER && workingEmoticon.length() < 10){
      println("appending key " + key);
      workingEmoticon += key;
    }
  }
}

class Character {

  // Put name, personality, etc info in here too
  String name = "";

  // Holds the EmoBranch (where on the dial) and strength (how far in/out)
  EmoState state = new EmoState();

  // Used for detecting loading new image and probably other things in the future
  ArrayList hist = new ArrayList();

  // location for drawing
  int xpos = width/2;
  int ypos = height/2;
  int speed = 3;
  
  int collX;
  int collY;
  int wid;

  Character(String name) {
    this.name = name;
  }
  
  Character(String name, int x, int y) {
    this.name = name;
    xpos = x;
    ypos = y;
  }

  public void saveState(){
    hist.add(state.copyCurrState());
  }
  //
  public void setEmoState(EmoState estate) {
    //saveState("");
    state = estate.copyCurrState();
  }
  
  public void setEmoState(int ID, String userText, int hap, int sad, int ang, int dis, int sur, int afr) {
    setEmoState(new EmoState(ID, userText, hap, sad, ang, dis, sur, afr));
  }
  
  public void updateCharacter(){
    wid = 15 * state.emoSize;
    collX = xpos - ((state.emoSize * 15)/2);
    collY = ypos - ((state.emoSize * 15)/2);
    //if (name == "pc") println(xpos + " - " + (state.emoSize/2));
  }
  
  public void drawCharacter(){
    /*
    noFill();
    stroke(255);
    rect(collX, collY, wid, wid);*/
    state.drawState(xpos, ypos);
  }
  
  public Boolean checkColl(int oX, int oY, int oW, int oH){
    if (oX + oW < collX || oX > collX + wid || oY + oH < collY || oY > collY + wid){
      return false;
    } else {
      return true;
    }
  }

}

class EmoState {

  int id = -1;
  String userText = "";

  // Width of the square emoticon
  int emoSize = 4;

  // 0 = happy
  // 1 = sad
  // 2 = angry
  // 3 = disgusted
  // 4 = surprised
  // 5 = afraid
  int[] emotions = new int[6];

  // Which emotions to show most powerfully
  int first = -1;
  int sec = -1;
  int third = -1;

  PImage eyes = null;
  PImage mouth = null;

  EmoState() {
    randomEmotion();
  }

  EmoState(int ID) {
    setEmoID(ID, "");
    randomEmotion();
  }

  EmoState(int ID, String userText, int hap, int sad, int ang, int dis, int sur, int afr) {
    setEmoID(ID, userText);
    setEmotion(hap, sad, ang, dis, sur, afr);
  }

  EmoState(int ID, String userText, int hap, int sad, int ang, int dis, int sur, int afr, PImage e, PImage m) {
    setEmoID(ID, userText);
    setEmotion(hap, sad, ang, dis, sur, afr);
    setImages(e, m);
  }

  public void randomEmotion() {
    // To make a variety of strengths
    int maxEmoStr = ((int) random(5)) +1;
    println("RANDOM arbitrary max emotion strength: " + maxEmoStr);
    for (int i = 0; i < emotions.length; i++) {
      emotions[i] = ((int) random(maxEmoStr)) + 1;
      println("emotion " + i + " set to " + emotions[i]);
    }
    configureFace();
  }

  public void configureFace() {
    ArrayList[] ranks = new ArrayList[emotions.length];
    // Initialize all these array Lists
    for (int i = 0; i < ranks.length; i++) {
      ranks[i] = new ArrayList();
    }
    for (int i = 0; i < emotions.length; i++) {
      // rank index = value of emotion, add the index of the emotion if it matches
      ranks[emotions[i]].add(i);
    }
    // Sort through the ranks to find the highests
    for (int i = ranks.length-1; i >= 0; i--) {
      int[] tempRanks = new int[3]; // indexes of top-ranking emotions
      tempRanks[0] = tempRanks[1] = tempRanks[2] = -1;
      if (ranks[i].size() >= 3) {
        // There are many options, pick 3 of them and randomize
        for (int j = 0; j < 3; j++) {
          int randIndex = (int) random(ranks[i].size());
          tempRanks[j] = (Integer)ranks[i].get(randIndex);
          //println("("+ i + ") tmpRank " + j + " set to index " + tempRanks[j]);
          ranks[i].remove(randIndex);
        }
      } 
      else if (ranks[i].size() == 2) {
        // There are two options, randomize their importance
        for (int j = 0; j < 2; j++) {
          int randIndex = (int) random(ranks[i].size());
          tempRanks[j] = (Integer)ranks[i].get(randIndex);
          //println("("+ i + ") tmpRank " + j + " set to index " + tempRanks[j]);
          ranks[i].remove(randIndex);
        }
      } 
      else if (ranks[i].size() == 1) {
        // There is one option, maximize its importance
        tempRanks[0] = (Integer)ranks[i].get(0);
        //println("("+ i + ") tmpRank " + 0 + " set to index " + tempRanks[0]);
      } 
      else {
        // Do nothing and move onto the next rank
      }
      for (int j = 0; j < tempRanks.length; j++) {
        if (first == -1 && tempRanks[j] != -1) {
          first = tempRanks[j];
          println("set first rank " + j + ": " + tempRanks[j] + " at value " + emotions[tempRanks[j]]);
        }
        else if (sec == -1 && tempRanks[j] != -1) {
          sec = tempRanks[j];
          println("set second rank " + j + ": " + tempRanks[j] + " at value " + emotions[tempRanks[j]]);
        } 
        else if (third == -1 && tempRanks[j] != -1) {
          third = tempRanks[j];
          println("set third rank " + j + ": " + tempRanks[j] + " at value " + emotions[tempRanks[j]]);
        }
      }
    }
    // Once it's all over, set the images!
    loadImages();
  }

  public void loadImages() {
    String eyeWords = "eyes_";
    String mouthWords = "mouth_";

    // Check if one emotion is accutely stronger than the others
    if (emotions[first] > 2 && emotions[first] - emotions[sec] >= 3) {
      println("-- Replacing sec emotion (" + emotions[sec] + ") with the first");
      sec = first;
    }

    // Pull out the files
    switch(first) {
      case(0):
      eyeWords += "happy";
      break;
      case(1):
      eyeWords += "sad";
      break;
      case(2):
      eyeWords += "angry";
      break;
      case(3):
      eyeWords += "disgusted";
      break;
      case(4):
      eyeWords += "surprised";
      break;
      case(5):
      eyeWords += "afraid";
      break;
    default:
      break;
    }
    switch(sec) {
      case(0):
      mouthWords += "happy";
      break;
      case(1):
      mouthWords += "sad";
      break;
      case(2):
      mouthWords += "angry";
      break;
      case(3):
      mouthWords += "disgusted";
      break;
      case(4):
      mouthWords += "surprised";
      break;
      case(5):
      mouthWords += "afraid";
      break;
    default:
      break;
    }

    eyeWords += "_"+ emotions[first];
    mouthWords += "_" + emotions[sec];
    println("LOADING: " + eyeWords + ".png");
    println("LOADING: " + mouthWords + ".png");
    setImages(loadImage(eyeWords + ".png"), loadImage(mouthWords + ".png"));
  }

  // 15 x 15 is the total size of the composite emoticon
  public void setImages(PImage eyes, PImage mouth) {
    this.eyes = eyes;
    eyes.resize(emoSize * 15, 0);
    this.mouth = mouth;
    mouth.resize(emoSize * 15, 0);
  }

  public void setEmotion(int hap, int sad, int ang, int dis, int sur, int afr) {
    emotions[0] = hap;
    emotions[1] = sad;
    emotions[2] = ang;
    emotions[3] = dis;
    emotions[4] = sur;
    emotions[5] = afr;
    configureFace();
  }

  public void setEmoID(int value, String userText) {
    this.id = value;
    this.userText = userText;
  }
  
  public void resize(int eSizeInt){
    emoSize = eSizeInt;
    loadImages();
  }

  public void drawState(int x, int y) {
    int imageX = x - ((emoSize*15)/2);
    int imageY = y - ((emoSize*15)/2);
    if (eyes != null) {
      image(eyes, imageX, imageY);
    }
    if (mouth != null) {
      image(mouth, imageX, imageY + eyes.height);
    }
  }

  public EmoState copyCurrState() {
    return new EmoState(id, userText, emotions[0], emotions[1], emotions[2], emotions[3], emotions[4], emotions[5], eyes, mouth);
  }
}

/*
Happy :) =) :D =D (: (= C: C= ^.^ ^_^ :P
 Sad :( =( D= D: ): )= T.T
 Angry >=( >=O >=o O=< )=< 
 Disgusted >=[ >=C D=< >:[ >:C D:< o_O :< >:
 Surprised :O =O :0 =0 :o =o o.o O.O
 Afraid ={ <.< >.> :{ X.X x.x o_o X_X x_x
 
 */

Character targetC = null;

public void parseInput() {
  println("parsing... " + workingEmoticon);
  if (workingEmoticon != "" && targetC != null) {
    String te = "";
    // remove white space
    for (int i = 0; i < workingEmoticon.length(); i++) {
      char c = workingEmoticon.charAt(i);
      if (c != ' ' && c != CODED) te += c;
    }
    println("Trimmed input at length " + te.length() + ": " + te);

    int emotionHit = -1;
    /*
    Boolean eyesDetect = false; // : or =
     Boolean openPara = false; // ( could be frown or smile depending
     Boolean closePara = false; // )
     Boolean dMouth = false; // D could be positve or negative
     Boolean cMouth = false; // C could be positive or negative
     Boolean pMouth = false; // P probably happy
     Boolean pointEyes = false; // ^ probably happy
     Boolean tearEyes = false; // T probably sad
     Boolean greaterT = false; // > angry or disgusted face
     Boolean lessT = false; // < angry or disgusted face
     Boolean oh = false; // "o" key, eye or mouth
     Boolean oH = false; // "O" capital "o"
     Boolean zero = false; // 0 number
     
     
     for (int i = 0; i < trimEmoticon.length(); i++){
     if (emotionHit == -1){
     char c = trimEmoticon.charAt(i);
     if (c == ':' || c == '='){ 
     // Keep eyes simple: seeing them doesn't necessarily means something is done
     eyesDetect = true;
     } else if (c == '('){ 
     openPara = true;
     if (eyesDetect == true) emotionHit = 1; // if Eyes come before mouth, we have hit sadness =(
     else emotionHit = 0; // Else, mouth comes first (:
     } else if (c == ')'){ 
     closePara = true;
     // if Eyes come before mouth, we have hit happiness =)
     if (eyesDetect == true) emotionHit = 0;
     } else if (c == 'D'){ dMouth = true;
     // if Eyes come before mouth, we have hit happiness =D
     if (eyesDetect == true) emotionHit = 0;
     else emotionHit = 1; // Else, mouth comes first D:
     } else if (c == 'C'){ cMouth = true;
     } else if (c == 'P'){ pMouth = true;
     } else if (c == '^'){ pointEyes = true;
     } else if (c == 'T'){ tearEyes = true;
     } else if (c == '>'){ greaterT = true;
     } else if (c == '<'){ lessT = true;
     } else if (c == 'o'){ oh = true;
     } else if (c == 'O'){ oH = true;
     } else if (c == '0'){ zero = true;
     }
     }*/

    if (te.compareTo(":)") == 0 || te.compareTo("=)") == 0 || te.compareTo(":D") == 0 || te.compareTo("=D") == 0 || te.compareTo("(:") == 0 ||
      te.compareTo("(=") == 0 || te.compareTo("C:") == 0 || te.compareTo("C=") == 0 || te.compareTo("^.^") == 0 || te.compareTo("^_^") == 0 || te.compareTo(":P") == 0 ||
      te.compareTo(":-)") == 0 || te.compareTo("=-)") == 0 || te.compareTo(":-D") == 0 || te.compareTo("=-D") == 0 || te.compareTo("(-:") == 0 ||
      te.compareTo("(-=") == 0 || te.compareTo("C-:") == 0 || te.compareTo("C-=") == 0 || te.compareTo(":-P") == 0) 
      emotionHit = 0;
    else if (te.compareTo(":(") == 0 || te.compareTo("=(") == 0 || te.compareTo("D=") == 0 || te.compareTo("D:") == 0 || 
      te.compareTo("):") == 0 || te.compareTo(")=") == 0 || te.compareTo("T.T") == 0 ||
      te.compareTo(":'(") == 0 || te.compareTo("='(") == 0 || te.compareTo("D'=") == 0 || te.compareTo("D':") == 0 || 
      te.compareTo(")':") == 0 || te.compareTo(")'=") == 0 ||
      te.compareTo(":,(") == 0 || te.compareTo("=,(") == 0 || te.compareTo("D,=") == 0 || te.compareTo("D,:") == 0 || 
      te.compareTo("),:") == 0 || te.compareTo("),=") == 0 ||
      te.compareTo(":.(") == 0 || te.compareTo("=.(") == 0 || te.compareTo("D.=") == 0 || te.compareTo("D.:") == 0 || 
      te.compareTo(").:") == 0 || te.compareTo(").=") == 0 ||
      te.compareTo(":-(") == 0 || te.compareTo("=-(") == 0 || te.compareTo("D-=") == 0 || te.compareTo("D-:") == 0 || 
      te.compareTo(")-:") == 0 || te.compareTo(")-=") == 0) 
      emotionHit = 1;
    else if (te.compareTo(">=(") == 0 || te.compareTo(">=O") == 0 || te.compareTo(">=o") == 0 || te.compareTo("O=<") == 0 || te.compareTo(")=<") == 0 ||
      te.compareTo(">=-(") == 0 || te.compareTo(">=-O") == 0 || te.compareTo(">=-o") == 0 || te.compareTo("O-=<") == 0 || te.compareTo(")-=<") == 0 ||
      te.compareTo(">:(") == 0 || te.compareTo(">:O") == 0 || te.compareTo(">:o") == 0 || te.compareTo("O:<") == 0 || te.compareTo("):<") == 0 ||
      te.compareTo(">:-(") == 0 || te.compareTo(">:-O") == 0 || te.compareTo(">:-o") == 0 || te.compareTo("O-:<") == 0 || te.compareTo(")-:<") == 0)
      emotionHit = 2;
    else if (te.compareTo(">=[") == 0 || te.compareTo(">=C") == 0 || te.compareTo("D=<") == 0 || 
      te.compareTo(">:[") == 0 || te.compareTo("]=<") == 0 || te.compareTo("]:<") == 0 ||
      te.compareTo(">:C") == 0 || te.compareTo("D:<") == 0 || te.compareTo("o_O") == 0 || 
      te.compareTo("O_o") == 0 || te.compareTo(":<") == 0 || te.compareTo(">:") == 0 || 
      te.compareTo("=<") == 0 || te.compareTo(">=") == 0 ||
      te.compareTo(">=-[") == 0 || te.compareTo(">=-C") == 0 || te.compareTo("D-=<") == 0 || 
      te.compareTo(">:-[") == 0 || te.compareTo("]-=<") == 0 || te.compareTo("]-:<") == 0 ||
      te.compareTo(">:-C") == 0 || te.compareTo("D-:<") == 0 || te.compareTo(":-<") == 0 || 
      te.compareTo(">-:") == 0 || te.compareTo("=-<") == 0 || te.compareTo(">-=") == 0)
      emotionHit = 3;
    else if (te.compareTo(":O") == 0 || te.compareTo("=O") == 0 || te.compareTo(":0") == 0 || te.compareTo("=0") == 0 ||
      te.compareTo(":o") == 0 || te.compareTo("=o") == 0 || te.compareTo("o.o") == 0 || te.compareTo("O.O") == 0 ||
      te.compareTo("O:") == 0 || te.compareTo("O=") == 0 || te.compareTo("0:") == 0 || 
      te.compareTo("0=") == 0 || te.compareTo("o:") == 0 || te.compareTo("o=") == 0 ||
      te.compareTo(":-O") == 0 || te.compareTo("=-O") == 0 || te.compareTo(":-0") == 0 || 
      te.compareTo("=-0") == 0 || te.compareTo(":-o") == 0 || te.compareTo("=-o") == 0 ||
      te.compareTo("O-:") == 0 || te.compareTo("O-=") == 0 || te.compareTo("0-:") == 0 || 
      te.compareTo("0-=") == 0 || te.compareTo("o-:") == 0 || te.compareTo("o-=") == 0)
      emotionHit = 4;
    else if (te.compareTo("={") == 0 || te.compareTo("}=") == 0 || te.compareTo("<.<") == 0 || 
      te.compareTo(">.>") == 0 || te.compareTo(":{") == 0 || te.compareTo("}:") == 0 ||
      te.compareTo("X.X") == 0 || te.compareTo("x.x") == 0 || te.compareTo("o_o") == 0 || te.compareTo("X_X") == 0 || te.compareTo("x_x") == 0 ||
      te.compareTo("=-{") == 0 || te.compareTo("}-=") == 0 || te.compareTo(":-{") == 0 || te.compareTo("}-:") == 0)
      emotionHit = 5;

    if (emotionHit != -1) {
      println("EMOTION DETECTED: " + emotionHit);
      targetC.saveState();
      for (int i = 0; i < targetC.state.emotions.length; i++) {
        if (i == emotionHit && targetC.state.emotions[i] < 5) {
          println("++ to emotion " + i + " for a total of " + (targetC.state.emotions[i]+1));
          targetC.state.emotions[i]++;
        } 
        else if (i != emotionHit && targetC.state.emotions[i] > 1) {
          
          int rand = (int) random(2) + 1;
          if (rand == 1) {
            targetC.state.emotions[i]--;
            println("RANDOM -- to emotion " + i + " for a total of " + (targetC.state.emotions[i]+1));
          }
          else {
            println("NO -- to emotion " + i + " for a total of " + (targetC.state.emotions[i]+1));
          }
        }
      }
      targetC.setEmoState(targetC.state.id, workingEmoticon, targetC.state.emotions[0], 
      targetC.state.emotions[1], targetC.state.emotions[2], targetC.state.emotions[3], 
      targetC.state.emotions[4], targetC.state.emotions[5]);
    }
    else {
      println("EMOTIONMISSED" + (workingEmoticon.compareTo(":)")) + "!!!!!!!!!!!!!");
    }
  }
}

  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "Emotale3" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
